/***************************************************************************
                          cache.cpp  -  description
                             -------------------
    begin                : Sat Sep 7 2002
    copyright            : (C) 2002 by Scott Wheeler
    email                : wheeler@kde.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <kstandarddirs.h>
#include <kdebug.h>

#include <qdir.h>

#include "cache.h"
#include "cachedtag.h"

Cache *Cache::m_cache = 0;

////////////////////////////////////////////////////////////////////////////////
// public methods
////////////////////////////////////////////////////////////////////////////////

Cache *Cache::instance()
{
    if(m_cache == 0) {
	m_cache = new Cache();
	m_cache->load();
    }
    return m_cache;
}

void Cache::save()
{
    QString dirName = KGlobal::dirs()->saveLocation("appdata");
    QString cacheFileName =  dirName + "cache.new";

    QFile f(cacheFileName);

    if(!f.open(IO_WriteOnly))
	return;

    QDataStream s(&f);

    for(QDictIterator<Tag>it(*this); it.current(); ++it) {
	s << it.current()->absFilePath()
	  << *(it.current());
    }

    f.close();
    QDir dir(dirName);
    dir.rename("cache.new", "cache");
}

////////////////////////////////////////////////////////////////////////////////
// protected methods
////////////////////////////////////////////////////////////////////////////////

Cache::Cache() : QDict<Tag>(m_cacheSize)
{

}

Cache::~Cache()
{
    delete m_cache;
}

void Cache::load()
{
    QString cacheFileName = KGlobal::dirs()->saveLocation("appdata") + "cache";

    QFile f(cacheFileName);

    if(!f.open(IO_ReadOnly))
	return;

    QDataStream s(&f);

    while(!s.atEnd()) {

	QString fileName;
	s >> fileName;

	CachedTag *t = new CachedTag(fileName);
	s >> *t;

	// Just do a dumb read from the cache.  Originally cache concistancy was
	// checked here, but this means that JuK was having to stat every file
	// in the cache while blocking GUI creation.  This has since been moved
	// to the event loop and is placed in the event loop in the 
	// CollectionListItem constructor.
    }

    f.close();
}
